CON
  _CLKMODE      = XTAL1 + PLL16X                        
  _XINFREQ      = 5_000_000


  ' ===================================
  '   change according to your setup!
  ' ===================================
  SDpinBase = 0

  NUM_OF_FILES = 3

obj
  sdfat: "fsrw"
  io:    "CogOS_IO(Term)"

VAR
  long i,j, tmp,tmp2,tmp3

PUB demo
  ' start the objects
  ' CogOS_IO(xxx) is an abstraction-layer or wrapper to map similar functions
  ' to ONE name. This way it's easy to exchange to a version that works with
  ' TV and keyboard ... or VGA and keyboard ... or FullDuplexSerial
  io.start( 0, 0 )

  ' wait until any key has been pushed. This way you can switch from Propeller Tool
  ' to PST without a hurry. At least during development all my terminal-based programs
  ' use this loop.
  tmp:=-1
  repeat while tmp==-1
    io.str( string( "Demo",$0d,"== please press key to start ==",$0d ) )
    tmp:=io.rxtime( 1000 )

  ' clear terminal screen and go home :o) (PST special)
  io.tx( 0 )
  io.tx( 1 )

  io.str( string("Mounting SD-card (") )
  tmp := \sdfat.mount( SDpinBase )
  io.dec( tmp )
  io.str( string( ")",13) )
  if tmp<0
    io.str( string( "Error mounting the SD card!" ) )
    repeat
    
  waitASec

  io.str( string("Find file_000.txt",13) )
  sdfat.vOpenSwap( 0, string( "file_000.txt" ) )
  waitASec

  io.str( string("Check wheather file_000.txt is initialized",13 ) )
  if sdfat.vPeek( 0, 0 )<>initPattern
    io.str( string("Initializing file_000.txt as DIR",13 ) )
    doInit 
  else
    io.str( string("Checking file_000.txt validity", 13 ) )
    doValidate

  io.str( string( "Ok ... using the files ...",13) )
  addString( 1, @aString )
  addString( 1, @aString )
  addString( 2, @aString )
  
  readNremove( 1, @rdBuffer )

  repeat

  
PUB addString( filenr, the_string )
  io.str( string( "adding string to file ") )
  io.dec( filenr )
  io.tx( 13 )
  
  ' calculate address of the file-entry in DIR
  tmp := filenr << 4
  ' get the sector address and the pointer to the end of used area
  tmp2 := sdfat.vPeek( 0, tmp )
  tmp3 := sdfat.vPeek( 0, tmp+4 )

  ' MISSING: first it should be checked wheather end of file has already been reached

  ' now set slot 1 to start-address of the file
  sdfat.vOpenSwapSector( 1, tmp2 )
  ' and write the string to the file
  sdfat.vwMemCopy( 1, tmp3, @aString, 64 )

  ' correct the value of the string-end pointer
  sdfat.vPoke( 0, tmp+4, tmp3+64 )

PUB readNremove( filenr, rd_buffer )
  ' calculate address of the file-entry in DIR
  tmp := filenr << 4
  ' get the sector address and the pointer to the end of used area
  tmp2 := sdfat.vPeek( 0, tmp )
  tmp3 := sdfat.vPeek( 0, tmp+4 )-64

  ' MISSING: check that the file has some content at all

  ' now set slot 1 to start-address of the file
  sdfat.vOpenSwapSector( 1, tmp2 )
  ' and read the string from file
  sdfat.vrMemCopy( 1, tmp3, rd_buffer, 64 )

  ' delete the string by adjusting the entry
  sdfat.vPoke( 0, tmp+4, tmp3 )
  ' and delete the content
  repeat 64
    sdfat.vbPoke( 1, tmp3++, $20 )

  io.str( string( "removing string: ") )
  io.str( rd_buffer )
  io.tx( 13 )
  
' this function loops over all files (starting from file_001.txt) and creates an
' entry in the DIR (file_000.txt)    
PUB doInit
  ' write the magic string to show in the next run, that the file is initialized
  sdfat.vPoke( 0,0,initPattern )
  
  ' just define the number of files on top of the code as constant
  repeat i from 1 to NUM_OF_FILES
    io.str( string("Try to popen file ") )
    io.dec( i )
    io.tx( "@" )
    waitASec
    
    ' open the file with a tweaked version of popen which returns the start sector
    tmp := sdfat.popen( fname, "r" )
    sdfat.pclose

    ' print sector number
    io.dec( tmp )
    io.tx( 13 )

    ' address of entry for this file is i*16, so we have room for 4 longs per entry
    tmp2 := i<<4
    ' store the sector
    sdfat.vPoke( 0, tmp2+=4 , tmp )
    ' store the filesize (just a dummy, to make it correct sdfat should be asked for the real size)
    sdfat.vPoke( 0, tmp2+=4, constant( 1024*1024 ) )
    ' initialize the number of used bytes
    sdfat.vPoke( 0, tmp2, 0 )

    ' this only works for one digit NUM_OF_FILES
    fname[7]++    


' this function is here to make the code more fool-proov. So, if the system has been shut down
' without flushing the DIR or in middle of write to a file, it'll recover with this function
PUB doValidate
  ' loop over all entries in the DIR - file (file_000.txt)
    ' read the start-sector, the size and the use counter
    ' attach actual file to slot 1
    ' check wheather the file contains zero at this location
      ' no, then do a sequential search for the real end
      ' store the real end in DIR
  
PUB waitASec
  waitcnt( clkfreq+cnt )

DAT
initPattern      byte   "DIR "
fname            byte   "file_001.txt"

aString          byte   "Just a simple fixed size test string", 0[30]
rdBuffer         byte   0[64]